library colorfilter_generator;

import 'package:flutter/material.dart';
import 'colorfilter-addons.dart';

///The [ColorFilterGenerator] class to define a Filter which will applied to each color, consists of multiple [SubFilter]s
class ColorFilterGenerator {
  String name;
  List<List<double>> subFilters;
  ColorFilterGenerator({required this.name}) : subFilters = [];

  Widget apply(Widget child) {
    Widget tree = child;

    for (int i = 0; i < subFilters.length; i++) {
      tree = ColorFiltered(
        colorFilter: ColorFilter.matrix(subFilters[i]),
        child: tree,
      );
    }

    return tree;
  }

  void addSubFilter(List<double> subFilter) {
    subFilters.add(subFilter);
  }

  void addSubFilters(List<List<double>> subFilters) {
    this.subFilters.addAll(subFilters);
  }
}

// NoFilter: No filter
class NoFilter extends ColorFilterGenerator {
  NoFilter() : super(name: "No Filter");
}

// Clarendon: adds light to lighter areas and dark to darker areas
class ClarendonFilter extends ColorFilterGenerator {
  ClarendonFilter() : super(name: "Clarendon") {
    addSubFilters([
      ColorFilterAddons.brightness(.1),
      ColorFilterAddons.contrast(.1),
      ColorFilterAddons.saturation(.15),
    ]);
  }
}

class AddictiveRedFilter extends ColorFilterGenerator {
  AddictiveRedFilter() : super(name: "AddictiveRed") {
    addSubFilters([
      ColorFilterAddons.addictiveColor(50, 0, 0),
    ]);
  }
}

class AddictiveBlueFilter extends ColorFilterGenerator {
  AddictiveBlueFilter() : super(name: "AddictiveBlue") {
    addSubFilters([
      ColorFilterAddons.addictiveColor(0, 0, 50),
    ]);
  }
}

// Gingham: Vintage-inspired, taking some color out
class GinghamFilter extends ColorFilterGenerator {
  GinghamFilter() : super(name: "Gingham") {
    addSubFilters([
      ColorFilterAddons.sepia(.04),
      ColorFilterAddons.contrast(-.15),
    ]);
  }
}

// Moon: B/W, increase brightness and decrease contrast
class MoonFilter extends ColorFilterGenerator {
  MoonFilter() : super(name: "Moon") {
    addSubFilters([
      ColorFilterAddons.grayscale(),
      ColorFilterAddons.contrast(-.04),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Lark: Brightens and intensifies colours but not red hues
class LarkFilter extends ColorFilterGenerator {
  LarkFilter() : super(name: "Lark") {
    addSubFilters([
      ColorFilterAddons.brightness(0.08),
      ColorFilterAddons.grayscale(),
      ColorFilterAddons.contrast(-.04),
    ]);
  }
}

// Reyes: a vintage filter, gives your photos a “dusty” look
class ReyesFilter extends ColorFilterGenerator {
  ReyesFilter() : super(name: "Reyes") {
    addSubFilters([
      ColorFilterAddons.sepia(0.4),
      ColorFilterAddons.brightness(0.13),
      ColorFilterAddons.contrast(-.05),
    ]);
  }
}

// Juno: Brightens colors, and intensifies red and yellow hues
class JunoFilter extends ColorFilterGenerator {
  JunoFilter() : super(name: "Juno") {
    addSubFilters([
      ColorFilterAddons.rgbScale(1.01, 1.04, 1),
      ColorFilterAddons.saturation(0.3),
    ]);
  }
}

// Slumber: Desaturates the image as well as adds haze for a retro, dreamy look – with an emphasis on blacks and blues
class SlumberFilter extends ColorFilterGenerator {
  SlumberFilter() : super(name: "Slumber") {
    addSubFilters([
      ColorFilterAddons.brightness(.1),
      ColorFilterAddons.saturation(-0.5),
    ]);
  }
}

// Crema: Adds a creamy look that both warms and cools the image
class CremaFilter extends ColorFilterGenerator {
  CremaFilter() : super(name: "Crema") {
    addSubFilters([
      ColorFilterAddons.rgbScale(1.04, 1, 1.02),
      ColorFilterAddons.saturation(-0.05),
    ]);
  }
}

// Ludwig: A slight hint of desaturation that also enhances light
class LudwigFilter extends ColorFilterGenerator {
  LudwigFilter() : super(name: "Ludwig") {
    addSubFilters([
      ColorFilterAddons.brightness(.05),
      ColorFilterAddons.saturation(-0.03),
    ]);
  }
}

// Aden: This filter gives a blue/pink natural look
class AdenFilter extends ColorFilterGenerator {
  AdenFilter() : super(name: "Aden") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(228, 130, 225, 0.13),
      ColorFilterAddons.saturation(-0.2),
    ]);
  }
}

// Perpetua: Adding a pastel look, this filter is ideal for portraits
class PerpetuaFilter extends ColorFilterGenerator {
  PerpetuaFilter() : super(name: "Perpetua") {
    addSubFilters([
      ColorFilterAddons.rgbScale(1.05, 1.1, 1),
    ]);
  }
}

// Amaro: Adds light to an image, with the focus on the centre
class AmaroFilter extends ColorFilterGenerator {
  AmaroFilter() : super(name: "Amaro") {
    addSubFilters([
      ColorFilterAddons.saturation(0.3),
      ColorFilterAddons.brightness(0.15),
    ]);
  }
}

// Mayfair: Applies a warm pink tone, subtle vignetting to brighten the photograph center and a thin black border
class MayfairFilter extends ColorFilterGenerator {
  MayfairFilter() : super(name: "Mayfair") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(230, 115, 108, 0.05),
      ColorFilterAddons.saturation(0.15),
    ]);
  }
}

// Rise: Adds a "glow" to the image, with softer lighting of the subject
class RiseFilter extends ColorFilterGenerator {
  RiseFilter() : super(name: "Rise") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 170, 0, 0.1),
      ColorFilterAddons.brightness(0.09),
      ColorFilterAddons.saturation(0.1),
    ]);
  }
}

// Hudson: Creates an "icy" illusion with heightened shadows, cool tint and dodged center
class HudsonFilter extends ColorFilterGenerator {
  HudsonFilter() : super(name: "Hudson") {
    addSubFilters([
      ColorFilterAddons.rgbScale(1, 1, 1.25),
      ColorFilterAddons.contrast(0.1),
      ColorFilterAddons.brightness(0.15),
    ]);
  }
}

// Valencia: Fades the image by increasing exposure and warming the colors, to give it an antique feel
class ValenciaFilter extends ColorFilterGenerator {
  ValenciaFilter() : super(name: "Valencia") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 225, 80, 0.08),
      ColorFilterAddons.saturation(0.1),
      ColorFilterAddons.contrast(0.05),
    ]);
  }
}

// X-Pro II: Increases color vibrance with a golden tint, high contrast and slight vignette added to the edges
class XProIIFilter extends ColorFilterGenerator {
  XProIIFilter() : super(name: "X-Pro II") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 255, 0, 0.07),
      ColorFilterAddons.saturation(0.2),
      ColorFilterAddons.contrast(0.15),
    ]);
  }
}

// Sierra: Gives a faded, softer look
class SierraFilter extends ColorFilterGenerator {
  SierraFilter() : super(name: "Sierra") {
    addSubFilters([
      ColorFilterAddons.contrast(-0.15),
      ColorFilterAddons.saturation(0.1),
    ]);
  }
}

// Willow: A monochromatic filter with subtle purple tones and a translucent white border
class WillowFilter extends ColorFilterGenerator {
  WillowFilter() : super(name: "Willow") {
    addSubFilters([
      ColorFilterAddons.grayscale(),
      ColorFilterAddons.colorOverlay(100, 28, 210, 0.03),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Lo-Fi: Enriches color and adds strong shadows through the use of saturation and "warming" the temperature
class LoFiFilter extends ColorFilterGenerator {
  LoFiFilter() : super(name: "Lo-Fi") {
    addSubFilters([
      ColorFilterAddons.contrast(0.15),
      ColorFilterAddons.saturation(0.2),
    ]);
  }
}

// Inkwell: Direct shift to black and white
class InkwellFilter extends ColorFilterGenerator {
  InkwellFilter() : super(name: "Inkwell") {
    addSubFilters([
      ColorFilterAddons.grayscale(),
    ]);
  }
}

// Hefe: Hight contrast and saturation, with a similar effect to Lo-Fi but not quite as dramatic
class HefeFilter extends ColorFilterGenerator {
  HefeFilter() : super(name: "Hefe") {
    addSubFilters([
      ColorFilterAddons.contrast(0.1),
      ColorFilterAddons.saturation(0.15),
    ]);
  }
}

// Nashville: Warms the temperature, lowers contrast and increases exposure to give a light "pink" tint – making it feel "nostalgic"
class NashvilleFilter extends ColorFilterGenerator {
  NashvilleFilter() : super(name: "Nashville") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(220, 115, 188, 0.12),
      ColorFilterAddons.contrast(-0.05),
    ]);
  }
}

// Stinson: washing out the colors ever so slightly
class StinsonFilter extends ColorFilterGenerator {
  StinsonFilter() : super(name: "Stinson") {
    addSubFilters([
      ColorFilterAddons.brightness(0.1),
      ColorFilterAddons.sepia(0.3),
    ]);
  }
}

// Vesper: adds a yellow tint that
class VesperFilter extends ColorFilterGenerator {
  VesperFilter() : super(name: "Vesper") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 225, 0, 0.05),
      ColorFilterAddons.brightness(0.06),
      ColorFilterAddons.contrast(0.06),
    ]);
  }
}

// Earlybird: Gives an older look with a sepia tint and warm temperature
class EarlybirdFilter extends ColorFilterGenerator {
  EarlybirdFilter() : super(name: "Earlybird") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 165, 40, 0.2),
      ColorFilterAddons.saturation(0.15),
    ]);
  }
}

// Brannan: Increases contrast and exposure and adds a metallic tint
class BrannanFilter extends ColorFilterGenerator {
  BrannanFilter() : super(name: "Brannan") {
    addSubFilters([
      ColorFilterAddons.contrast(0.2),
      ColorFilterAddons.colorOverlay(140, 10, 185, 0.1),
    ]);
  }
}

// Sutro: Burns photo edges, increases highlights and shadows dramatically with a focus on purple and brown colors
class SutroFilter extends ColorFilterGenerator {
  SutroFilter() : super(name: "Sutro") {
    addSubFilters([
      ColorFilterAddons.brightness(-0.1),
      ColorFilterAddons.saturation(-0.1),
    ]);
  }
}

// Toaster: Ages the image by "burning" the centre and adds a dramatic vignette
class ToasterFilter extends ColorFilterGenerator {
  ToasterFilter() : super(name: "Toaster") {
    addSubFilters([
      ColorFilterAddons.sepia(0.1),
      ColorFilterAddons.colorOverlay(255, 145, 0, 0.2),
    ]);
  }
}

// Walden: Increases exposure and adds a yellow tint
class WaldenFilter extends ColorFilterGenerator {
  WaldenFilter() : super(name: "Walden") {
    addSubFilters([
      ColorFilterAddons.brightness(0.1),
      ColorFilterAddons.colorOverlay(255, 255, 0, 0.2),
    ]);
  }
}

// 1977: The increased exposure with a red tint gives the photograph a rosy, brighter, faded look.
class F1977Filter extends ColorFilterGenerator {
  F1977Filter() : super(name: "1977") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 25, 0, 0.15),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Kelvin: Increases saturation and temperature to give it a radiant "glow"
class KelvinFilter extends ColorFilterGenerator {
  KelvinFilter() : super(name: "Kelvin") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 140, 0, 0.1),
      ColorFilterAddons.rgbScale(1.15, 1.05, 1),
      ColorFilterAddons.saturation(0.35),
    ]);
  }
}

// Maven: darkens images, increases shadows, and adds a slightly yellow tint overal
class MavenFilter extends ColorFilterGenerator {
  MavenFilter() : super(name: "Maven") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(225, 240, 0, 0.1),
      ColorFilterAddons.saturation(0.25),
      ColorFilterAddons.contrast(0.05),
    ]);
  }
}

// Ginza: brightens and adds a warm glow
class GinzaFilter extends ColorFilterGenerator {
  GinzaFilter() : super(name: "Ginza") {
    addSubFilters([
      ColorFilterAddons.sepia(0.06),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Skyline: brightens to the image pop
class SkylineFilter extends ColorFilterGenerator {
  SkylineFilter() : super(name: "Skyline") {
    addSubFilters([
      ColorFilterAddons.saturation(0.35),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Dogpatch: increases the contrast, while washing out the lighter colors
class DogpatchFilter extends ColorFilterGenerator {
  DogpatchFilter() : super(name: "Dogpatch") {
    addSubFilters([
      ColorFilterAddons.contrast(0.15),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Brooklyn
class BrooklynFilter extends ColorFilterGenerator {
  BrooklynFilter() : super(name: "Brooklyn") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(25, 240, 252, 0.05),
      ColorFilterAddons.sepia(0.3),
    ]);
  }
}

// Helena: adds an orange and teal vibe
class HelenaFilter extends ColorFilterGenerator {
  HelenaFilter() : super(name: "Helena") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(208, 208, 86, 0.2),
      ColorFilterAddons.contrast(0.15),
    ]);
  }
}

// Ashby: gives images a great golden glow and a subtle vintage feel
class AshbyFilter extends ColorFilterGenerator {
  AshbyFilter() : super(name: "Ashby") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 160, 25, 0.1),
      ColorFilterAddons.brightness(0.1),
    ]);
  }
}

// Charmes: a high contrast filter, warming up colors in your image with a red tint
class CharmesFilter extends ColorFilterGenerator {
  CharmesFilter() : super(name: "Charmes") {
    addSubFilters([
      ColorFilterAddons.colorOverlay(255, 50, 80, 0.12),
      ColorFilterAddons.contrast(0.05),
    ]);
  }
}

List<ColorFilterGenerator> presetFiltersList = [
  NoFilter(),
  AddictiveBlueFilter(),
  AddictiveRedFilter(),
  AdenFilter(),
  AmaroFilter(),
  AshbyFilter(),
  BrannanFilter(),
  BrooklynFilter(),
  CharmesFilter(),
  ClarendonFilter(),
  CremaFilter(),
  DogpatchFilter(),
  EarlybirdFilter(),
  F1977Filter(),
  GinghamFilter(),
  GinzaFilter(),
  HefeFilter(),
  HelenaFilter(),
  HudsonFilter(),
  InkwellFilter(),
  JunoFilter(),
  KelvinFilter(),
  LarkFilter(),
  LoFiFilter(),
  LudwigFilter(),
  MavenFilter(),
  MayfairFilter(),
  MoonFilter(),
  NashvilleFilter(),
  PerpetuaFilter(),
  ReyesFilter(),
  RiseFilter(),
  SierraFilter(),
  SkylineFilter(),
  SlumberFilter(),
  StinsonFilter(),
  SutroFilter(),
  ToasterFilter(),
  ValenciaFilter(),
  VesperFilter(),
  WaldenFilter(),
  WillowFilter(),
  XProIIFilter(),
];
